"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnumTypeAdapter = void 0;
var breeze_client_1 = require("breeze-client");
var ts_odatajs_1 = require("ts-odatajs");
var models_1 = require("../models");
var utilities_1 = require("../utilities");
var DefaultIsFlags = 'false';
var DefaultUnderlyingType = 'Edm.Int32';
var TrueValue = 'true';
var EnumTypeAdapter = (function () {
    function EnumTypeAdapter() {
    }
    EnumTypeAdapter.prototype.adapt = function (metadata) {
        ts_odatajs_1.oData.utils.forEachSchema(metadata.schema, this.adaptSchema.bind(this));
    };
    EnumTypeAdapter.prototype.adaptSchema = function (schema) {
        var enumTypes = schema.enumType || [];
        enumTypes.forEach(this.adaptEnumType, this);
    };
    EnumTypeAdapter.prototype.adaptEnumType = function (enumType) {
        var _this = this;
        var _a, _b;
        (_a = enumType.isFlags) !== null && _a !== void 0 ? _a : (enumType.isFlags = DefaultIsFlags);
        (_b = enumType.underlyingType) !== null && _b !== void 0 ? _b : (enumType.underlyingType = DefaultUnderlyingType);
        var enumName = enumType.name;
        var underlyingType = enumType.underlyingType.split('.').pop();
        var dataType = utilities_1.Utilities.getDataType(underlyingType);
        var isFlags = enumType.isFlags.toLowerCase() === TrueValue;
        var enumValue = new models_1.EdmEnum({
            isFlags: isFlags,
            name: enumName,
            underlyingDataType: dataType
        });
        enumValue._$typeName = breeze_client_1.DataType.prototype.name;
        enumType.member.forEach(function (member) { return _this.adaptEnumMember(enumValue, member); });
        breeze_client_1.DataType[enumName] = new breeze_client_1.DataType(enumValue);
    };
    EnumTypeAdapter.prototype.adaptEnumMember = function (enumValue, member) {
        var options = {
            name: member.name,
            displayName: member.displayName,
            rawValue: member.value
        };
        enumValue.addSymbol(options);
    };
    return EnumTypeAdapter;
}());
exports.EnumTypeAdapter = EnumTypeAdapter;
//# sourceMappingURL=enum-type-adapter.js.map