"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OData4PredicateVisitor = void 0;
var breeze_client_1 = require("breeze-client");
var OData4PredicateVisitor = (function () {
    function OData4PredicateVisitor() {
    }
    OData4PredicateVisitor.initialize = function () {
        var visitor = new OData4PredicateVisitor();
        breeze_client_1.Predicate.prototype.toODataFragment = function (context) {
            return this.visit.call(this, context, visitor);
        };
    };
    OData4PredicateVisitor.prototype.passthruPredicate = function () {
        return this.value;
    };
    OData4PredicateVisitor.prototype.unaryPredicate = function (context) {
        var predVal = this.pred.visit(context);
        var op = this.op.key;
        return "".concat(op, " (").concat(predVal, ")");
    };
    OData4PredicateVisitor.prototype.binaryPredicate = function (context) {
        var expr1Val = this.expr1.visit(context);
        var expr2Val = this.expr2.visit(context);
        var prefix = context.prefix;
        if (prefix) {
            expr1Val = "".concat(prefix, "/").concat(expr1Val);
        }
        var op = this.op.key;
        if (op === 'in') {
            var result = expr2Val
                .map(function (v) { return "(".concat(expr1Val, " eq ").concat(v, ")"); })
                .join(' or ');
            return result;
        }
        else if (this.op.isFunction) {
            return "".concat(op, "(").concat(expr1Val, ",").concat(expr2Val, ")");
        }
        else {
            return "".concat(expr1Val, " ").concat(op, " ").concat(expr2Val);
        }
    };
    OData4PredicateVisitor.prototype.andOrPredicate = function (context) {
        var op = this.op.key;
        var result = this.preds
            .map(function (pred) { return "(".concat(pred.visit(context), ")"); })
            .join(" ".concat(op, " "));
        return result;
    };
    OData4PredicateVisitor.prototype.anyAllPredicate = function (context) {
        var op = this.op.key;
        var exprVal = this.expr.visit(context);
        if (!this.pred.op) {
            return "".concat(exprVal, "/").concat(op, "()");
        }
        var prefix = context.prefix;
        var idx = 1;
        if (prefix) {
            exprVal = "".concat(prefix, "/").concat(exprVal);
            idx = Number(prefix.substring(1)) + 1;
        }
        prefix = "x".concat(idx);
        var newContext = __assign(__assign({}, context), { entityType: this.expr.dataType, prefix: prefix });
        var newPredVal = this.pred.visit(newContext);
        return "".concat(exprVal, "/").concat(op, "(").concat(prefix, ": ").concat(newPredVal, ")");
    };
    OData4PredicateVisitor.prototype.litExpr = function () {
        var _this = this;
        if (Array.isArray(this.value)) {
            return this.value
                .map(function (v) { return _this.dataType.fmtOData(v); });
        }
        else {
            return this.dataType.fmtOData(this.value);
        }
    };
    OData4PredicateVisitor.prototype.propExpr = function (context) {
        var entityType = context.entityType;
        return entityType
            ? entityType.clientPropertyPathToServer(this.propertyPath, '/')
            : this.propertyPath;
    };
    OData4PredicateVisitor.prototype.fnExpr = function (context) {
        var exprVals = this.exprs
            .map(function (expr) { return expr.visit(context); });
        return "".concat(this.fnName, "(").concat(exprVals, ")");
    };
    return OData4PredicateVisitor;
}());
exports.OData4PredicateVisitor = OData4PredicateVisitor;
//# sourceMappingURL=breeze-odata4-predicateVisitor.js.map